/*
 * Copyright (C) 2006 Terry Stenvold
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "pz.h"
#include <errno.h>

#define MET  16
#define IMP  17
#define STK  18
#define ANG  19
#define BLK  24
#define DP   25
#define STY  20
#define NTY  21
#define OOE  22
#define OTF  23

#define METRIC   25.4
#define IMPERIAL 0.03937
#define STYY     0.866
#define NTYY     0.500
#define OOEE     0.300
#define OTFF     0.207

#define N_BUTTON (20 - 1)
struct _button {
	char *ch;
	unsigned char math;
} button[] = {
	{"7", 7}, {"8", 8}, {"9", 9}, {"Me", MET}, {"60", STY},
	{"4", 4}, {"5", 5}, {"6", 6}, {"Im", IMP}, {"90", NTY},
	{"1", 1}, {"2", 2}, {"3", 3}, {"St", STK}, {"118", OOE},
	{"0", 0}, {".", DP},{"Clr", BLK},{"An",ANG}, {"135", OTF},
};

static PzModule *module;
static double number;
static signed char e;
static unsigned char op;
static int cur_button;
static ttk_surface image;
static int imgw, imgh;

static void draw_button(PzWidget *wid, ttk_surface srf, int row, int col)
{
#define XOFF (wid->w / 8)
#define YOFF (wid->h / 4)
#define SECW ((wid->w - XOFF*2)/5) 
#define SECH ((wid->h - YOFF - wid->h/8)/4) 
#define BPAD ((SECW - ttk_text_width(ttk_textfont, "8"))/4)
#define BW (SECW - BPAD)
#define BH (SECH - BPAD)
	char *text[] = {"button.selected.fg", "button.default.fg",
		"button.selected.bg", "button.default.bg",
		"button.selected.border", "button.default.border"};

	int pos = (row * 5) + col;
	unsigned char ap = !(pos == cur_button);
	int x = (XOFF + BPAD/2) + col*(BW + BPAD);
	int y = (YOFF + BPAD/2) + row*(BH + BPAD);

	ttk_ap_fillrect(srf, ttk_ap_get(text[ap+2]), x+1, y+1, x+BW-1, y+BH-1);
	ttk_ap_rect(srf, ttk_ap_get(text[ap+4]), x, y, x + BW, y + BH);
	ttk_text(srf, ttk_textfont, x+(BW-(BW/2)-ttk_text_width(ttk_textfont, "8"))/2,
			y+(BH-ttk_text_height(ttk_textfont))/2,
			ttk_ap_getx(text[ap])->color, button[pos].ch);
}

static void calculator_draw(PzWidget *wid, ttk_surface srf)
{
	int row, col;
	int x = XOFF + BPAD/2;
	int y = wid->h/8;
	int xe = x + (wid->w - XOFF*2) - BPAD;
	char s[24];

	//ttk_blit_image (image, srf, wid->x + (wid->w - imgw) / 2, wid->y + (wid->h - imgh) / 2);
	snprintf(s, 24, "%g", number);
	ttk_rect(srf, x, y, xe, y+BH, ttk_ap_getx("window.fg")->color);
	ttk_text(srf, ttk_textfont, xe - ttk_text_width(ttk_textfont, s) - 1,
			y + (BH - ttk_text_height(ttk_textfont))/2,
			ttk_ap_getx("window.fg")->color, s);
	
	for (row = 0; row < 4; row++)
		for (col = 0; col < 5; col++)
			draw_button(wid, srf, row, col);
}

static double lpow(double x, signed char p)
{
	double i = x;
	if (p > 0) while (--p > 0) x *= i;
	else if (p-- < 0) while (p++ < 0) x /= i;
	else return 1;
	return x;
}

static double operation(unsigned char op, double a)
{
	double radang;

	switch (op) {
	case MET: return METRIC * a;
	case IMP:return IMPERIAL * a;
	case STY: return STYY * a;
	case NTY:return NTYY * a;
	case OOE: return OOEE * a;
	case OTF:return OTFF * a;
	case STK: 
		radang = ( 3.141592654 / 180 ) * a; //turns angle from dec to radian
     		a= 5 * (sin (radang)); //calculates for 5 inch
		return a;
	case ANG:  
		a = asin( a / 5 ); //calculates for 5 inch 
		a =   a / (3.141592654 / 180 ) ; //turns angle from radian to decimal
		return a;
	}
	return 0;
}

static void calculate(int pos)
{
	struct _button *p = &button[pos];
	static int stump = 1;
	op = p->math;
	if (p->math & 16) {
		switch (p->math) {
		case MET:
		case IMP:
		case STY: 
		case NTY:
		case OOE: 
		case OTF:
		case STK: 
		case ANG:
			if (op) 
			number = operation(op, number);
			op = p->math;
			stump = 0;
			e = 0;
			break;
		case BLK:
			number = operation(op, number);
			op = 0;
			stump = 0;
			e = 0;
			break;
		case DP:
			if (!e) --e;
			break;
		}
	}
	else if (!stump) {
		if (!e) number *= 10;
		number += p->math * lpow(10, e);
		if (e < 0) --e;
	}
	else {
		number = p->math;
		stump = 0;
	}
}

#define BWRAP(x,y,z) if (x < y) x = z
#define TWRAP(x,y,z) if (x > z) x = y
#define WRAP(x,y,z) do { BWRAP(x,y,z); TWRAP(x,y,z); } while(0)

static int calculator_events(PzEvent *e)
{
	int ret = TTK_EV_CLICK;

	switch(e->type) {
	case PZ_EVENT_BUTTON_DOWN:
		switch (e->arg) {
		case PZ_BUTTON_ACTION:
			calculate(cur_button);
			e->wid->dirty = 1;
			break;

		case PZ_BUTTON_MENU:
			pz_close_window(e->wid->win);
			break;

		default:
			ret = TTK_EV_UNUSED;
		}
		break;

	case PZ_EVENT_SCROLL:
		TTK_SCROLLMOD(e->arg, 5);
		cur_button += e->arg;
		WRAP(cur_button, 0, N_BUTTON);
		e->wid->dirty = 1;
		break;

	default:
		ret = TTK_EV_UNUSED;
		break;
	}
	return ret;
}

static PzWindow *new_calculator()
{
	PzWindow *ret;
	number = 0; 
	e = 0;
	cur_button = 0;
	op = 0;

	//pz_message_title ("Instructions" , " Blk: Clears screen\n 60-135: Drill point depth\n An: Angle from stack hieght\n St: Stack hieght from angle\n Met: Converts to metric\n Imp: Converts to Imperial "); Now in a text file	

	//image = ttk_load_image (pz_module_get_datapath (module, "sine.png"));
	
    //if (!image) {
	//pz_error ("Could not load %s: %s", pz_module_get_datapath (module, "sine.png"), 
	//	  strerror (errno));
	//return TTK_MENU_DONOTHING;
   // } ttk_surface_get_dimen (image, &imgw, &imgh);


	ret = pz_new_window(("Machinist Calc"), PZ_WINDOW_NORMAL);
	pz_add_widget(ret, calculator_draw, calculator_events);

	return pz_finish_window(ret);
}

static void init_calculator()
{	
	
	module = pz_register_module("mach", NULL);
	pz_menu_add_action("Extras/Utilities/Machinist", new_calculator); 
}

PZ_MOD_INIT(init_calculator)
